import { Component } from '@angular/core';
import { DialogComponent, DialogService } from "ng2-bootstrap-modal";
import { Archivo } from '../../shared/objects/archivo';
import { TranslateService } from '@ngx-translate/core';

export interface ConfirmModel {
  nombre:string;
  descripcion:string;
  parentContext: any;
  esDirectorio:boolean;
}
@Component({  
    selector: 'confirm',
    template: `<div class="modal-dialog" style="margin-top:100px;">
                <div class="modal-content">

                   <div class="modal-header">
                      <h6 class="modal-title" *ngIf="esDirectorio">
                        {{ "i18n.action.new" | translate | titleCase }}
                        {{ "i18n.object.folder" | translate | titleCase }}
                      </h6> 
                      <h6 class="modal-title" *ngIf="!esDirectorio">
                        {{ "i18n.action.new" | translate | titleCase }}
                        {{ "i18n.object.file" | translate | titleCase }}
                      </h6> 
                      <button type="button" class="close" (click)="close()" style="margin-left:8px;">&times;</button>
                   </div>
                   
                   <div class="modal-body">
                    <form>
                      <div class="form-group">
                        <label for="recipient-name" class="form-control-label">
                          {{ "i18n.object.name" | translate | titleCase }}:
                        </label>
                        <input type="text" class="form-control" id="recipient-name" [(ngModel)]="nombre" [ngModelOptions]="{standalone: true}" >
                      </div>
                      <div class="form-group" *ngIf="esDirectorio">
                        <label for="message-text" class="form-control-label">
                          {{ "i18n.object.descr" | translate | titleCase }}:
                        </label>
                        <textarea class="form-control" id="message-text" [(ngModel)]="descripcion" [ngModelOptions]="{standalone: true}" ></textarea>
                      </div>
                    </form>
                  </div>

                  <div class="modal-footer">
                    <button type="button" class="btn btn-success" (click)="confirm()">
                      {{ "i18n.action.create" | translate | titleCase }}
                    </button>
                  </div>

                 </div>
              </div>`
})
export class NuevoArchivo extends DialogComponent<ConfirmModel, boolean> implements ConfirmModel {
  nombre: string;
  esDirectorio: boolean;
  descripcion: string;
  parentContext: any;
  translateService: any;

  constructor(dialogService: DialogService, public translate: TranslateService) {
    super(dialogService);
    this.translateService = translate;
  }
  confirm() {
    var nombre = this.nombre;
    var desc = this.descripcion;
    var archivo : Archivo;
    archivo = new Archivo();
    archivo.cedulaCreador = this.parentContext.directorioActual.cedulaCreador;
    if(this.esDirectorio){
      archivo.contenido = desc;
    } else {
      archivo.contenido = '';
    }
    archivo.directorio = this.esDirectorio;
    archivo.editable = true;
    archivo.fechaCreacion  = new Date();
    archivo.nombre = nombre;
    archivo.padreId =  this.parentContext.directorioActual.id;
    var that = this.parentContext;
    var regex = /^[A-Z]/
    if(regex.test(nombre)){
      this.parentContext.haskellService.crearArchivo(archivo).subscribe(
          archivo => {
            var id = that.directorioActual.id;
            that.recargarArchivos(id);        
          }, 
          error => {
              this.parentContext.notifService.error(error);
          });
      this.close();
    }else{
        alert(this.translateService.get('i18n.warning.file.capitalLetter').value);
    }
  }
  
}